/*
    Fichier trtentete.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence.fr

    Traductions des droits d'utilisation dans les fichiers :
    licence.de , licence.en , licence.es , licence.it
    licence.nl , licence.pt , licence.eo , licence.eo-utf


    Bibliothque de fonctions permettant de

    - reprer les champs de l'entte du mail
    - convertir dans ces champs les accents et caractres spciaux
      cods sur 7 bits grace  l'encodage quoted-printable
    - mmoriser les lignes de l'entte aprs conversion
*/

#include <stdio.h>
#include "buflect.h"
#include "encodage.h"
#include "trtentete.h"



/* cherche si la dernire ligne lue commence par un mot cl particulier */

int start (char *motcle)
{
    int i;

    i = 0;

    // on teste caractre par caractre en ignorant la casse
    while (motcle [i] && tolower (buf_lect [i]) == tolower (motcle [i]))
        i++;

    // le mot cl doit tre suivi de  :  dans la ligne lue
    return (buf_lect [i] == ':');
}



/* convertit la ligne d'entte en interprtant les caractres spciaux */

void majlignentete ()
{
    int encodage;    // mode d'encodage
    int charset;     // jeu de caractres
    int carcode;     // caractre correspondant  un codage quoted-printable
    int decalsuiv;   // indicateur pour conversion de caractre encod Utf8
    int decode_;     // indicateur pour conversion '_' en ' '
    char carcourant; // caractre contenant une valeur en base64
    char valcar [4]; // valeur aprs conversion des caractres encods base64
    int i, j, k;     // compteurs


    // initialisation
    i = 0;
    j = 0;
    encodage = defaut;
    charset  = defaut;
    decode_  = 1;

    // exploration de toute la ligne
    while (buf_lect [i] != '\0')
    {
        // si dbut de mot d'encodage
        if (buf_lect [i] == '=' && buf_lect [i+1] == '?' && encodage == defaut)
        {
            // se positionner sur le charset
            i = i + 2;

            // identification simplifie du charset (sur 1 caractre)
            if (tolower (buf_lect [i]) == 'i')
                charset = Iso8859;
            else if (tolower (buf_lect [i]) == 'u')
            {
                charset = Utf8;
                decalsuiv = 0;  // initialisation indicateur de conversion
            }
            else
                charset = xCharset;  // sera trait comme ISO-8859

            // aller  la fin du charset
            do
                i++;
            while (buf_lect [i] && buf_lect [i] != '?');

            // se positionner sur le caractre d'encodage
            i++;

            // identification du mode d'encodage
            if (tolower (buf_lect [i]) == 'q')
                encodage = QuotedPrint;
            else if (tolower (buf_lect [i]) == 'b')
                encodage = Base64;
            else
                encodage = xBits;

            // aller au dbut de la zone encode
            i = i + 2;
        }

        // si fin de zone encode
        if (buf_lect [i] == '?' && buf_lect [i+1] == '=' && encodage != defaut)
        {
            // sauter le marqueur de fin
            i = i + 2;

            // rinitialiser les indicateurs d'tat
            charset  = defaut;
            encodage = defaut;
        }

        // si caractre quoted printable  convertir
        else if (buf_lect [i] == '=' && encodage == QuotedPrint
                 && ((buf_lect [i+1] >= '0' && buf_lect [i+1] <= '9')
                  || (buf_lect [i+1] >= 'A' && buf_lect [i+1] <= 'F')))
        {
            // on rcupre le caractre correspondant
            carcode = hexa (buf_lect + ++i);

            // si encodage classique
            if (charset != Utf8)
            {
                // si ce caractre n'est pas un caractre de controle
                if (carcode > 0x1F)
                    // le mmoriser
                    buf_lect [j++] = carcode;

                // sinon, si caractre autre que CR
                else if (carcode != '\r')
                    // mettre un blanc pour limiter les problmes d'affichage
                    buf_lect [j++] = ' ';
            }
            else
            {
                // si caractre  on convertira le caractre qui suit
                if (carcode == 0xC3)
                    decalsuiv = 1;

                // on recopie les caractres autres que  et CR
                else if (carcode != 0xC2 && carcode != '\r')
                {
                    if (decalsuiv)
                    {
                        // en faisant une conversion si ncessaire
                        buf_lect [j++] = carcode | 0x40;
                        decalsuiv = 0;
                    }
                    else
                        buf_lect [j++] = carcode;
                }
            }

            // se positionner sur le caractre suivant
            i = i + 2;
        }

        // sinon si encodage base64 en cours
        else if (encodage == Base64)
        {
            // dcoder une srie de 4 caractres
            for (k = 0; k < 4; k++)
            {
                // rcuprer un caractre
                carcourant = buf_lect [i++];

                // chercher sa valeur sur 6 bits et la mmoriser
                if ('A' <= carcourant && carcourant <= 'Z')
                    valcar [k] = carcourant - 'A';
                else if ('a' <= carcourant && carcourant <= 'z')
                    valcar [k] = carcourant + 26 - 'a';
                else if ('0' <= carcourant && carcourant <= '9')
                    valcar [k] = carcourant + 52 - '0';
                else if (carcourant == '+')
                    valcar [k] = 62;
                else if (carcourant == '/')
                    valcar [k] = 63;
            }

            // reconstituer les 3 caractres dcods correspondants
            buf_lect [j++] = (valcar [0] << 2) | (valcar [1] >> 4);

            // s'il y avait un = , a faisait moins de 3 caractres  obtenir
            if (buf_lect [i-2] != '=')
            {
                buf_lect [j++] = (valcar [1] << 4) | (valcar [2] >> 2);

                if (carcourant != '=')
                    buf_lect [j++] = (valcar [2] << 6) | (valcar [3]);
            }
        }

        // sinon, si on a le caractre spcial _
        else if (buf_lect [i] == '_' && decode_)
        {
            // le remplacer par un blanc
            buf_lect [j++] = ' ';

            // et passer au caractre suivant
            i++;
        }
        // sinon (cas gnral)
        else
        {
            // prendre en compte les < et > pour viter de
            // convertir les _ en ' ' dans une adresse email
            if (buf_lect [i] == '<')
                decode_ = 0;
            else if (buf_lect [i] == '>')
                decode_ = 1;

            // sinon on conserve le caractre courant et on passe au suivant
            // remarque : on n'a pas jug utile de convertir l'encodage base64
            buf_lect [j++] = buf_lect [i++];
        }
    }

    // terminaison de la chaine de caractres
    buf_lect [j] = '\0';
}



/* convertit la ligne lue en interprtant les caractres
   spciaux et la mmorise dans le buffer de destination */

void memconvbuf (char *bufdest)
{
    // on convertit les caractres spciaux
    majlignentete ();

    // mmorisation de la ligne aprs conversion
    membuf (bufdest);
}



/* mmorise la ligne lue en dans le buffer de destination */

void membuf (char *bufdest)
{
    int i;

    i = 0;

    // copie tronque  la longueur max
    do
        bufdest [i] = buf_lect [i++];
    while (i < 79 && buf_lect [i] != '\0' && buf_lect [i] != '\n');

    // terminaison de la chaine de caractres
    bufdest [i] = '\0';
}



/* retourne le code hexa lu  la position courante */

int hexa (char *chaine)
{
    int valcar;
    char souschaine [3];

    souschaine [0] = chaine [0];
    souschaine [1] = chaine [1];
    souschaine [2] = '\0';

    sscanf (souschaine, "%x", &valcar);
    return valcar;
}
